@@ -50,6 +50,9 @@ class TestKernelClient(TestCase):
         kc = self.kc
         msg_id = kc.history(session=0)
         self.assertIsInstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        kc._recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = kc.history(session=0, reply=True, timeout=TIMEOUT)
         self._check_reply("history", reply)
 
@@ -57,6 +60,9 @@ class TestKernelClient(TestCase):
         kc = self.kc
         msg_id = kc.inspect("who cares")
         self.assertIsInstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        kc._recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = kc.inspect("code", reply=True, timeout=TIMEOUT)
         self._check_reply("inspect", reply)
 
@@ -64,6 +70,9 @@ class TestKernelClient(TestCase):
         kc = self.kc
         msg_id = kc.complete("who cares")
         self.assertIsInstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        kc._recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = kc.complete("code", reply=True, timeout=TIMEOUT)
         self._check_reply("complete", reply)
 
@@ -71,6 +80,9 @@ class TestKernelClient(TestCase):
         kc = self.kc
         msg_id = kc.kernel_info()
         self.assertIsInstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        kc._recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = kc.kernel_info(reply=True, timeout=TIMEOUT)
         self._check_reply("kernel_info", reply)
 
@@ -78,6 +90,9 @@ class TestKernelClient(TestCase):
         kc = self.kc
         msg_id = kc.comm_info()
         self.assertIsInstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        kc._recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = kc.comm_info(reply=True, timeout=TIMEOUT)
         self._check_reply("comm_info", reply)
 
@@ -150,36 +165,54 @@ class TestAsyncKernelClient:
     async def test_history(self, kc):
         msg_id = kc.history(session=0)
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.history(session=0, reply=True, timeout=TIMEOUT)
         self._check_reply("history", reply)
 
     async def test_inspect(self, kc):
         msg_id = kc.inspect("who cares")
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.inspect("code", reply=True, timeout=TIMEOUT)
         self._check_reply("inspect", reply)
 
     async def test_complete(self, kc):
         msg_id = kc.complete("who cares")
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.complete("code", reply=True, timeout=TIMEOUT)
         self._check_reply("complete", reply)
 
     async def test_is_complete(self, kc):
         msg_id = kc.is_complete("who cares")
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.is_complete("code", reply=True, timeout=TIMEOUT)
         self._check_reply("is_complete", reply)
 
     async def test_kernel_info(self, kc):
         msg_id = kc.kernel_info()
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.kernel_info(reply=True, timeout=TIMEOUT)
         self._check_reply("kernel_info", reply)
 
     async def test_comm_info(self, kc):
         msg_id = kc.comm_info()
         assert isinstance(msg_id, str)
+        # Drain the first reply to avoid race condition
+        await kc._async_recv_reply(msg_id, timeout=TIMEOUT)
+        # Now test the reply=True convenience path
         reply = await kc.comm_info(reply=True, timeout=TIMEOUT)
         self._check_reply("comm_info", reply)
 
